library IEEE;
library work;

use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use work.image1_pkg.ALL;

entity image1_top is
   generic(g_NUMBER_OF_CHANNELS : NATURAL := 6);
   port (
      FPGA_CLK_P     : in STD_LOGIC;
      FPGA_CLK_N     : in STD_LOGIC;
      led_pw_o       : out STD_LOGIC;
      led_err_o      : out STD_LOGIC;
      led_ttl_o      : out STD_LOGIC;
      fpga_o_en      : out STD_LOGIC;
      fpga_o_ttl_en  : out STD_LOGIC;
      fpga_o_inv_en  : out STD_LOGIC;
      fpga_o_blo_en  : out STD_LOGIC;
      level          : in  STD_LOGIC;
      switch_i       : in  STD_LOGIC;   --! General enable
      manual_rst_n_o : out STD_LOGIC;   --! It allows power sequencing of the
                                        --! 24V rail after a security given
                                        --! delay
      pulse_i_front  : in  STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
      pulse_o_front  : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
      pulse_i_rear   : in  STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
      pulse_o_rear   : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
      led_o_front    : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
      led_o_rear     : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
      --! This LED will show the status of the PLL
      led_link_up_o  : out STD_LOGIC;
      --! WR LEDs not to let them ON
      led_pps_o      : out STD_LOGIC;
      led_wr_ok_o    : out STD_LOGIC;
      inv_i          : in  STD_LOGIC_VECTOR(4 downto 1);
      inv_o          : out STD_LOGIC_VECTOR(4 downto 1);
      --! Lines for the i2c_slave
      SCL_I          : in  STD_LOGIC;
      SCL_O          : out STD_LOGIC;
      SCL_OE         : out STD_LOGIC;
      SDA_I          : in  STD_LOGIC;
      SDA_O          : out STD_LOGIC;
      SDA_OE         : out STD_LOGIC;
      FPGA_GA        : in  STD_LOGIC_VECTOR(4 downto 0);
      --! RTM identifiers, should match with the expected values
      --! TODO: add matching
      FPGA_RTMM      : in STD_LOGIC_VECTOR(3 downto 0);
      FPGA_RTMP      : in STD_LOGIC_VECTOR(3 downto 0);
      --! Pins of the SPI interface to write into the Flash memory
      FPGA_PROM_CCLK    : out STD_LOGIC;
      FPGA_PROM_CSO_B_N : out STD_LOGIC;
      FPGA_PROM_DIN     : in  STD_LOGIC;
      FPGA_PROM_MOSI    : out STD_LOGIC);
end image1_top;


architecture Behavioral of image1_top is

   --! i2c_slave signals
   signal wb_i2c_master_stb    : STD_LOGIC;
   signal wb_i2c_master_cyc    : STD_LOGIC;
   signal wb_i2c_master_sel    : STD_LOGIC_VECTOR(xx downto 0);
   signal wb_i2c_master_we     : STD_LOGIC;
   signal wb_i2c_master_data_i : STD_LOGIC_VECTOR(31 downto 0);
   signal wb_i2c_master_data_o : STD_LOGIC_VECTOR(31 downto 0);
   signal wb_i2c_master_addr_o : STD_LOGIC_VECTOR(xx downto 0);
   signal wb_i2c_master_ack    : STD_LOGIC;
   signal wb_i2c_master_rty    : STD_LOGIC;
   signal wb_i2c_master_err    : STD_LOGIC;
   signal wb_i2c_slave_stb     : STD_LOGIC;
   signal wb_i2c_slave_cyc     : STD_LOGIC;
   signal wb_i2c_slave_sel     : STD_LOGIC_VECTOR(xx downto 0);
   signal wb_i2c_slave_we      : STD_LOGIC;
   signal wb_i2c_slave_data_i  : STD_LOGIC_VECTOR(31 downto 0);
   signal wb_i2c_slave_data_o  : STD_LOGIC_VECTOR(31 downto 0);
   signal wb_i2c_slave_addr_i  : STD_LOGIC_VECTOR(xx downto 0);
   signal wb_i2c_slave_ack     : STD_LOGIC;
   signal wb_i2c_slave_rty     : STD_LOGIC;
   signal wb_i2c_slave_err     : STD_LOGIC;

   --! m25p32 signals
   signal wb_m25p32_we         : STD_LOGIC;
   signal wb_m25p32_stb        : STD_LOGIC;
   signal wb_m25p32_cyc        : STD_LOGIC;
   signal wb_m25p32_sel        : STD_LOGIC_VECTOR(xx downto 0);
   signal wb_m25p32_data_i     : STD_LOGIC_VECTOR(31 downto 0);
   signal wb_m25p32_data_o     : STD_LOGIC_VECTOR(31 downto 0);
   signal wb_m25p32_addr_i     : STD_LOGIC_VECTOR(xx downto 0);
   signal wb_m25p32_ack        : STD_LOGIC;
   signal wb_m25p32_rty        : STD_LOGIC;
   signal wb_m25p32_err        : STD_LOGIC;

begin

   inst_basic_trigger: basic_trigger_top
   port map(FPGA_CLK_P     => FPGA_CLK_P,
            FPGA_CLK_N     => FPGA_CLK_N,
            led_pw_o       => led_pw_o,
            led_err_o      => led_err_o,
            led_ttl_o      => led_ttl_o,
            fpga_o_en      => fpga_o_en,
            fpga_o_ttl_en  => fpga_o_ttl_en,
            fpga_o_inv_en  => fpga_o_inv_en,
            fpga_o_blo_en  => fpga_o_blo_en,
            level          => level,
            switch_i       => switch_i,
            manual_rst_n_o => manual_rst_n_o,
            pulse_i_front  => pulse_i_front,
            pulse_o_front  => pulse_o_front,
            pulse_i_rear   => pulse_i_rear,
            pulse_o_rear   => pulse_o_rear,
            led_o_front    => led_o_front,
            led_o_rear     => led_o_rear,
            led_link_up_o  => led_link_up_o,
            led_pps_o      => led_pps_o,
            led_wr_ok_o    => led_wr_ok_o,
            inv_i          => inv_i,
            inv_o          => inv_o);

   --! We are always the slave so we disable writes into SCL pin
   SCL_OEN <= '0';

   inst_i2c_slave: i2c_slave_top
   port map(sda_oen           => SDA_OE,
            sda_i             => SDA_I,
            sda_o             => SDA_O,
            scl_oen           => open,
            scl_i             => SCL_I,
            scl_o             => SCL_O,
            wb_clk            => clk_i,
            wb_rst_i          => FPGA_SYSRESET_N,
            wb_master_stb_o   => wb_i2c_master_stb,
            wb_master_cyc_o   => wb_i2c_master_cyc,
            wb_master_sel_o   => wb_i2c_master_sel,
            wb_master_we_o    => wb_i2c_master_we,
            wb_master_data_i  => wb_i2c_master_data_i,
            wb_master_data_o  => wb_i2c_master_data_o,
            wb_master_addr_o  => wb_i2c_master_addr_o,
            wb_master_ack_i   => wb_i2c_master_ack,
            wb_master_rty_i   => wb_i2c_master_rty,
            wb_master_err_i   => wb_i2c_master_err,
            wb_slave_stb_i    => wb_i2c_slave_stb,
            wb_slave_cyc_i    => wb_i2c_slave_cyc,
            wb_slave_sel_i    => wb_i2c_slave_sel,
            wb_slave_we_i     => wb_i2c_slave_we,
            wb_slave_data_i   => wb_i2c_slave_data_i,
            wb_slave_data_o   => wb_i2c_slave_data_o,
            wb_slave_addr_i   => wb_i2c_slave_addr_i,
            wb_slave_ack_o    => wb_i2c_slave_ack,
            wb_slave_rty_o    => wb_i2c_slave_rty,
            wb_slave_err_o    => wb_i2c_slave_err,
            pf_wb_addr_o      => open,
            rd_done_o         => open,
            wr_done_o         => open,
            i2c_addr_i        => "00"&FPGA_GAP);

   inst_m25p32: m25p32_top
   port map(wb_rst_i          => FPGA_SYSRESET_N,
            wb_clk            => clk_i,
            wb_we_i           => wb_m25p32_we,
            wb_stb_i          => wb_m25p32_stb,
            wb_cyc_i          => wb_m25p32_cyc,
            wb_sel_i          => wb_m25p32_sel,
            wb_data_i         => wb_m25p32_data_i,
            wb_data_o         => wb_m25p32_data_o,
            wb_addr_i         => wb_m25p32_addr_i,
            wb_ack_o          => wb_m25p32_ack,
            wb_rty_o          => wb_m25p32_rty,
            wb_err_o          => wb_m25p32_err,
            miso_word_rcv     => open,
            op_finished_o     => open,
            prom_mosi_o       => FPGA_PROM_MOSI,
            prom_cclk_o       => FPGA_PROM_CCLK,
            prom_cs0_b_n_o    => FPGA_PROM_CSO_B_N,
            prom_din_i        => FPGA_PROM_DIN);

end Behavioral;
